// Persistence of Vision RayTracer version 3.1 Scene description file
// File:        MATERIAL.LIB
// Description: The material library file, called from all model libraries.
//
//              Copyright Anton Raves.
//
// Date:        2008/10/12
// Author:      Anton Raves
//
// Version:     0515

#ifndef (maquette)
  #declare maquette = 0;        // By default maquette mode is switched off
#end
#ifndef (radio)
  #declare radio = 0;           // By default radiosity is switched off
#end
#ifndef (setting)
  #declare setting = 1;
#end
#declare release = val(str(version,3,1));
// Declaration of pixel ratios for the camera
// ------------------------------------------
#declare computer =   4 / 3;    // setting for computer monitors 640 x 480
#declare hdtv     =   16 / 9;   // setting for 16:9 widescreen televisions 640 x 360
// Declarations for standard measurements
// using these will place Lego(TM) pieces
// correctly AND will produce goodlooking
// seems between them.
// --------------------------------------
#declare flat = 3.2;            // standard offset to place a piece on top of a flat piece
#declare high = 9.6;            // standard offset to place a pice on top of a regular piece
#declare wide = 8;
// Declaration of pieces often used
// --------------------------------
#macro axle_ending(age)
  difference {
    merge {
      box { <-3.900002, -0.900002, -1.6> <0.000001, 0.900002, 1.6> }
      box { <-3.900001, -1.6, -0.900002> <0.000002, 1.6, 0.900002> }
      cylinder { -0.900001*z, 0.900001*z, 0.750001 translate <-3.149999, -1.599999, 0> }
      cylinder { -0.900001*z, 0.900001*z, 0.750001 translate <-3.149999, 1.599999, 0> }
      cylinder { -0.900001*y, 0.900001*y, 0.750001 translate <-3.149999, 0, -1.599999> }
      cylinder { -0.900001*y, 0.900001*y, 0.750001 translate <-3.149999, 0, 1.599999> }
    }
    difference {
      box { <-4, -4, -4> <1, 4, 4> }
      cylinder { -4.1*x, 1.1*x, 2.35 }
    }
    #if (age = 2)
      difference {
        cylinder { -0.15*x, 0*x, 0.85 }
        torus { 0.85, 0.15 rotate 90*z }
        translate <-3.75, 0, 0>
      }
    #end
  }
#end

#macro axle_opening(long, age)
  difference {
    cylinder { 0*x, 1*x, 2.45 }
    cylinder { -0.1*x, 1.1*x, 0.5 translate <0, 1.6, 1.6> }
    cylinder { -0.1*x, 1.1*x, 0.5 translate <0, -1.6, -1.6> }
    box { <-0.1, 1.6, 1> <1.1, 3, 3> }
    box { <-0.1, 1, 1.6> <1.1, 3, 3> }
    box { <-0.1, -3, -3> <1.1, -1, -1.6> }
    box { <-0.1, -3, -3> <1.1, -1.6, -1> }
    #if (age = 1)
      cylinder { -0.1*x, 1.1*x, 0.5 translate <0, 1.6, -1.6> }
      cylinder { -0.1*x, 1.1*x, 0.5 translate <0, -1.6, 1.6> }
      box { <-0.1, 1.6, -1> <1.1, 3, -3> }
      box { <-0.1, 1, -3> <1.1, 3, -1.6> }
      box { <-0.1, -3, 1.6> <1.1, -1, 3> }
      box { <-0.1, -3, 1> <1.1, -1.6, 3> }
    #else
      cylinder { -0.1*x, 1.1*x, 0.25 translate <0, 1.6, -1.6> }
      cylinder { -0.1*x, 1.1*x, 0.25 translate <0, -1.6, 1.6> }
      box { <-0.1, 1.6, -0.25> <1.1, 3, 0.25> rotate -45*x }
      box { <-0.1, -1.6, -0.25> <1.1, -3, 0.25> rotate -45*x }
    #end
    scale <long, 1, 1>
  }
#end

#macro axle_section(long)
  difference {
    merge {
      box { <0, -0.9, -3> <1, 0.9, 3> }
      box { <-0.000001, -3, -0.9> <1.000001, 3, 0.9> }
    }
    difference {
      box { <-0.1, -4, -4> <1.1, 4, 4> }
      cylinder { -0.2*x, 1.2*x, 2.35 }
    }
    scale <long, 1, 1>
  }
#end

// This object can be used to create different clothes for the little people,
// just merge it with a body part after setting the texture for the paint
// seperately.
#macro body_paint()
difference {
  box { <-7.499999, 0.000001, -3.900005> <7.499999, 12.399999, -3.85> }
  box { <-10, -2, -5> <0, 20, 5> rotate -10*z translate <-7.499997, 1, 0> }
  box { <0, -2, -5> <10, 20, 5> rotate 10*z translate <7.499997, 1, 0> }
}
#end

#macro conductive_button()
  merge {
    difference {
      merge {
        cylinder { 3*y, 4.65*y, 2.4 }
        cylinder { 4*y, 4.8*y, 2.25 }
        torus { 2.25, 0.15 translate <0, 4.65, 0> }
        #if (setting > 3)
          object { llogo() scale <0.65, 0.65, 0.65> translate <0, 4.8, 0> }
        #end
      }
      box { <-1, 1, -3> <3, 4.4, 1> }
      difference {
        box { <-3, 1, -5> <3, 6, -0.75> }
        cylinder { 2*y, 5*y, 2 }
        rotate -45*y
      }
    }
    intersection {
      cylinder { 2.75*y, 4.3*y, 2.3 }
      box { <-1, 1, -3> <3, 5, 1> }
      texture { chrome_tex }
    }
  }
#end

#macro high_open_rounded_button()
  #if (setting > 2)
    difference {
      merge {
        cylinder { 3*y, 11.05*y, 2.4 }
        cylinder { 4*y, 11.2*y, 2.25 }
        torus { 2.25, 0.15 translate <0, 11.05, 0> }
      }
      cylinder { 2*y, 12*y, 1.6 }
    }
  #else
    difference {
      cylinder { 3*y, 11.2*y, 2.4 }
      cylinder { 2*y, 12*y, 1.6 }
    }
  #end
#end
#macro high_ring()
difference {
  cylinder { 0*y, 8.7*y, 3.2 }
  cylinder { -1*y, 9*y, 2.6 }
}
#end
#macro llogo()
  merge {
    sphere { <-0.826, 0, -1.344>, 0.084 }
    cylinder { <-0.826, 0, -1.344>, <0.826, 0, -1.708>, 0.084 }
    sphere { <0.826, 0, -1.708>, 0.084 }
    cylinder { <0.826, 0, -1.708>, <0.826, 0, -1.176>, 0.084 }
    sphere { <0.826, 0, -1.176>, 0.084 }
    sphere { <-0.826, 0, -0.504>, 0.084 }
    cylinder { <-0.826, 0, -0.504>, <-0.826, 0, 0.014>, 0.084 }
    sphere { <-0.826, 0, 0.014>, 0.084 }
    cylinder { <0, 0, -0.686>, <0, 0, -0.35>, 0.084 }
    sphere { <0, 0, -0.35>, 0.084 }
    sphere { <0.826, 0, -0.868>, 0.084 }
    cylinder { <0.826, 0, -0.868>, <0.826, 0, -0.336>, 0.084 }
    sphere { <0.826, 0, -0.336>, 0.084 }
    cylinder { <-0.826, 0, -0.504>, <0.826, 0, -0.8682>, 0.084 }
    sphere { <-0.5033, 0, 0.798>, 0.084 }
    torus { 0.2583, 0.084 clipped_by { plane { <0.56, 0, -0.126>, 0 } } translate <-0.56, 0, 0.546> }
    cylinder { <-0.6167, 0, 0.294>, <0.5033, 0, 0.042>, 0.084 }
    torus { 0.2583, 0.084 clipped_by { plane { <-0.56, 0, 0.126>, 0 } } translate <0.56, 0, 0.294> }
    cylinder { <0.6167, 0, 0.546>, <0, 0, 0.686>, 0.084 }
    sphere { <0, 0, 0.686>, 0.084 }
    cylinder { <0, 0, 0.686>, <0, 0, 0.476>, 0.084 }
    sphere { <0, 0, 0.476>, 0.084 }
    torus { 0.2583, 0.084 clipped_by { plane { <0.56, 0, -0.126>, 0 } } translate <-0.56, 0, 1.386> }
    cylinder { <-0.6167, 0, 1.134>, <0.5033, 0, 0.882>, 0.084 }
    torus { 0.2583, 0.084 clipped_by { plane { <-0.56, 0, 0.126>, 0 } } translate <0.56, 0, 1.134> }
    cylinder { <0.6167, 0, 1.386>, <-0.5033, 0, 1.638>, 0.084 }
  }
#end
#macro open_rounded_button()
  #if (setting > 2)
    difference {
      merge {
        cylinder { 3*y, 4.65*y, 2.4 }
        cylinder { 4*y, 4.8*y, 2.25 }
        torus { 2.25, 0.15 translate <0, 4.65, 0> }
      }
      cylinder { 2*y, 5*y, 1.6 }
    }
  #else
    difference {
      cylinder { 3*y, 4.8*y, 2.4 }
      cylinder { 2*y, 5*y, 1.6 }
    }
  #end
#end
#macro pin(age)
  #if (age = 1)
    cylinder { 0*y, 2.1*y, 1.5 }
  #else
    difference {
      cylinder { 0*y, 2.1*y, 1.5 }
      cylinder { -1*y, 3*y, 0.7 }
    }
  #end
#end
#macro ring()
difference {
  cylinder { 0*y, 2.1*y, 3.2 }
  cylinder { -1*y, 3*y, 2.6 }
}
#end
#macro rounded_button()
  merge {
    cylinder { 3*y, 4.65*y, 2.4 }
    cylinder { 4*y, 4.8*y, 2.25 }
    torus { 2.25, 0.15 translate <0, 4.65, 0> }
    #if (setting > 3)
      object { llogo() translate <0, 4.8, 0> }
    #end 
  }
#end
#switch (release)

  #case (3.6)
    //#debug "\nUsing 3.6 textures\n"
    #declare basic_finish = finish { ambient 0 diffuse 0.95 brilliance 1 phong 0 phong_size 40 reflection 0 }
    // Declaration of the colors used
    // ------------------------------
    #declare t_blue = color rgb <0.1, 0.3, 0.75>;
    #declare t_cyan = color rgb <0.725, 0.95, 0.975>;
    #declare t_green = color rgb <0.025, 0.7, 0.025>;
    #declare t_dark_green = color rgb <0.05, 0.7, 0.05>*.6;
    #declare _red = color rgb <0.9, 0.025, 0.025>;
    #declare t_yellow = color rgb <1, 0.95, 0.05>;
    #declare beige = color rgb <0.85, 0.775, 0.5>;
    #declare black = color rgb <0.05, 0.05, 0.05>;
    #declare t_black = color rgbt <0.015, 0.01, 0.005, 0.65>;
    #declare _blue = color rgb <0.1, 0.3, 0.975>;
    #declare _blue2 = color rgb <0.1, 0.2, 0.6>;
    #declare lt_blue = color rgb <0.3, 0.7, 1>;
    #declare bnsf_green = color rgb <0.015, 0.15, 0.015>;
    #declare brown = color rgb <0.4, 0.2, 0>;
    #declare brown2 = color rgb <0.7, 0.4, 0.2>;
    #declare chrome = color rgb <0.15, 0.15, 0.15>;
    #declare cyan = color rgb <0.5, 0.825, 1>;
    #declare dark_blue = color rgb <0.05, 0.1, 0.4>;
    #declare dark_cyan = color rgb <0.3, 0.45, 0.75>;
    #declare dark_red = color rgb <0.5, 0.025, 0.025>;
    #declare dark_grey = color rgb <0.1, 0.1, 0.1>;
    #declare dark_grey2 = color rgb <0.45, 0.45, 0.45>;
    #declare _green = color rgb <0.025, 0.5, 0.025>;
    #declare _green2 = color rgb <0.025, 0.7, 0.025>;
    #declare lt_green = color rgb <0, 0, 0>;
    #declare gold = color rgb <0.95, 0.85, 0>;
    #declare grey = color rgb <0.55, 0.55, 0.55>;
    #declare grey2 = color rgb <0.35, 0.35, 0.35>;
    #declare grey3 = color rgb <0.8, 0.8, 0.8>;
    #declare lt_grey = color rgb <0.575, 0.575, 0.625>;
    #declare mint = color rgb <0.01, 0.8, 0.575>;
    #declare orange = color rgb <1, 0.5, 0.15>;
    #declare orange_bright = color rgb <1, 0.35, 0.15>;
    #declare purple = color rgb <0.4, 0.05, 0.4>;
    #declare white = color rgb <1, 1, 1>;
    #declare yellow = color rgb <0.975, 0.775, 0.025>;
    #declare yellowgreen = color rgb <0.6, 1, 0.15>;
    #declare t_yellowgreen = color rgb <0.85, 0.95, 0.45>;
    #declare t_purple = color rgb <0.4, 0.05, 0.4>;
    // Declaration of the finishes used
    // --------------------------------
    #if (radio)
      #if (setting < 2)
        #declare bumping =
        normal { bumps 0 }
        #declare lego_finish =
        finish { ambient 0.2 specular 1 roughness 0.005 reflection 0 }
        #declare silverish_finish =
        finish { ambient 0.2 specular 1 roughness 0.005 reflection 0 }
        #declare t_lego_finish =
        finish { ambient 0.3 specular 1 roughness 0.005 reflection 0 }
      #else
        #declare bumping =
        normal { bumps 0.3 scale 0.05 }
        #declare lego_finish =
        finish { ambient 0.2 specular 1 roughness 0.005 reflection 0.05 }
        #declare silverish_finish =
        finish { ambient 0 specular 1 roughness 0.005 metallic brilliance 0.75 reflection 0.05 }
        #declare t_lego_finish =
        finish { ambient 0 specular 1 roughness 0.005 reflection 0.05 }
      #end
    #else
      #if (setting < 2)
        #declare bumping =
        normal { bumps 0 }
        #declare lego_finish =
        finish { ambient 0.4 specular 1 roughness 0.005 reflection 0 }
        /////////////////////////////////////////////////////////////////////////////////////////////// low no rad
        #declare silverish_finish =
        finish { ambient 0.4 specular 1 roughness 0.005 reflection 0 }
        #declare t_lego_finish =
        finish { ambient 0.6 specular 1 roughness 0.005 reflection 0 }
      #else
        #declare bumping =
        normal { bumps 0.3 scale 0.05 }
        #declare lego_finish =
        finish { //ambient 0.08 phong .3 phong_size 450 roughness 0.05 reflection 0.05 } // road finish
         ambient 0.4 specular 1 roughness 0.005 reflection 0.05 }
        ///////////////////////////////////////////////////////////////////////////////// Lego FINISH!!!!!!!!!!!!!
        #declare silverish_finish =
        finish { ambient 0.4 specular 1 roughness 0.005 metallic brilliance 0.75 reflection 0.05 }
        #declare t_lego_finish =
        finish { ambient 0.6 specular 1 roughness 0.005 reflection 0.05 }
      #end
    #end
    // The normal colored textures themselves...
    // --------------------------
    #declare beige_tex = texture { pigment { color beige } finish { lego_finish } }
    #declare black_tex = texture { pigment { color black } finish { lego_finish } }
    #declare blue_tex = texture { pigment { color _blue } finish { lego_finish } }
    #declare blue2_tex = texture { pigment { color _blue2 } finish { lego_finish } }
    #declare bnsf_green_tex = texture { pigment { color bnsf_green } finish { lego_finish } }
    #declare brown_tex = texture { pigment { color brown } finish { lego_finish } }
    #declare brown2_tex = texture { pigment { color brown2 } finish { lego_finish } }
    #declare cyan_tex = texture { pigment { color cyan } finish { lego_finish } }
    #declare dark_blue_tex = texture { pigment { color dark_blue } finish { lego_finish } }
    #declare dark_cyan_tex = texture { pigment { color dark_cyan } finish { lego_finish } }
    #declare dark_grey2_tex = texture { pigment { color dark_grey2 } finish { lego_finish } }
    #declare dark_red_tex = texture { pigment { color dark_red } finish { lego_finish } }
    #declare green_tex = texture { pigment { color _green } finish { lego_finish } }
    #declare green2_tex = texture { pigment { color _green2 } finish { lego_finish } }
    #declare lt_green_tex = texture { pigment { color lt_green } finish { lego_finish } }
    #declare grey_tex = texture { pigment { color grey } finish { lego_finish } }
    #declare grey2_tex = texture { pigment { color grey2 } finish { lego_finish } }
    #declare lt_grey_tex = texture { pigment { color lt_grey } finish { lego_finish } }
    #declare mint_tex = texture { pigment { color mint } finish { lego_finish } }
    #declare orange_tex = texture { pigment { color orange } finish { lego_finish } }
    #declare orange_bright_tex = texture { pigment { color orange_bright } finish { lego_finish ambient 0.4 } }
    #declare purple_tex = texture { pigment { color purple } finish { lego_finish } }
    #declare red_tex = texture { pigment { color _red } finish { lego_finish } }
    #declare white_tex = texture { pigment { color white } finish { lego_finish } }
    #declare yellow_tex = texture { pigment { color yellow } finish { lego_finish } }
    #declare yellowgreen_tex = texture { pigment { color yellowgreen } finish { lego_finish } }
    // The transparent textures...
    // ---------------------------
    #if (setting < 2)
      #declare t_black_tex = texture { pigment { color t_black filter 0 } finish { t_lego_finish } }
      #declare t_blue_tex = texture { pigment { color t_blue filter 0 } finish { t_lego_finish } }
      #declare t_clear_tex = texture { pigment { color white filter 0 } finish { t_lego_finish } }
      #declare t_cyan_tex = texture { pigment { color t_cyan filter 0 } finish { t_lego_finish } }
      #declare t_green_tex = texture { pigment { color t_green filter 0 } finish { t_lego_finish } }
      #declare t_dark_green_tex = texture { pigment { color t_dark_green filter 0 } finish { t_lego_finish } }
      #declare t_old_clear_tex = texture { pigment { color white filter 0 } finish { lego_finish } }
      #declare t_orange_tex = texture { pigment { color orange filter 0 } finish { t_lego_finish } }
      #declare t_red_tex = texture { pigment { color _red filter 0 } finish { t_lego_finish } }
      #declare t_yellow_tex = texture { pigment { color t_yellow filter 0 } finish { t_lego_finish } }
      #declare t_yellowgreen_tex = texture { pigment { color t_yellowgreen filter 0 } finish { t_lego_finish } }
      #declare t_white_tex = texture { pigment { color white filter 0 } finish { t_lego_finish } }
      #declare t_vizor_tex = texture { pigment { color white filter 0.9 } finish { t_lego_finish } } 
    #else
      #declare t_black_tex = texture { pigment { color t_black filter 1 } finish { t_lego_finish } }
      #declare t_blue_tex = texture { pigment { color t_blue filter 1 } finish { t_lego_finish } }
      #declare t_clear_tex = texture { pigment { color white filter 1 } finish { t_lego_finish } }
      #declare t_cyan_tex = texture { pigment { color t_cyan filter 1 } finish { t_lego_finish } }
      #declare t_green_tex = texture { pigment { color t_green filter 1 } finish { t_lego_finish } }
      #declare t_dark_green_tex = texture { pigment { color t_dark_green transmit .5 filter .4 } finish { t_lego_finish } }
      #declare t_old_clear_tex = texture { pigment { color white filter 0.5 } finish { lego_finish } }
      #declare t_orange_tex = texture { pigment { color orange filter 0.85 } finish { t_lego_finish } }
      #declare t_red_tex = texture { pigment { color _red filter 1 } finish { t_lego_finish } }
      #declare t_yellow_tex = texture { pigment { color t_yellow filter 1 } finish { t_lego_finish } }
      #declare t_yellowgreen_tex = texture { pigment { color t_yellowgreen filter 0.85 } finish { t_lego_finish ambient 0.2 } }
      #declare t_white_tex = texture { pigment { color white filter 1 } finish { t_lego_finish } }
      #declare t_vizor_tex = texture { pigment { color white filter 0.9 } finish { t_lego_finish } }
      
      
    #end
    // And the other ones...
    // ---------------------
    #declare blue_gold_tex = texture { pigment { color _blue } finish { lego_finish metallic reflection 0.5 brilliance 6 } }
    #declare chrome_tex = texture { pigment { color chrome } finish { lego_finish metallic reflection 0.5 brilliance 6 } }
    #declare conductive_tex = texture { pigment { color grey } finish { silverish_finish reflection .1 } }
    #declare copperish_tex = texture { pigment { color brown2 } finish { silverish_finish } }
    #declare gold_tex = texture { pigment { color gold } finish { lego_finish metallic reflection 0.5 brilliance 6 } }
    #declare goldish_tex = texture { pigment { color gold } finish { silverish_finish } }
    #declare hologram_tex = texture { pigment { color grey2 } finish { lego_finish metallic reflection 0.5 brilliance 6 } }
    #declare lt_blue_tex = texture { pigment { color lt_blue filter 0.05 } finish { lego_finish reflection 0 } }
    #declare r_black_tex = texture { pigment { color dark_grey } finish { lego_finish reflection .01 } }
    #declare r_brown_tex = texture { pigment { color brown } finish { lego_finish reflection .01 } }
    #declare r_grey_tex = texture { pigment { color grey } finish { lego_finish reflection .01 } }
    #declare r_white_tex = texture { pigment { color white } finish { lego_finish reflection .01 } }
    #declare r_yellow_tex = texture { pigment { color yellow } finish { lego_finish reflection .01 } }
    #declare silverish_tex = texture { pigment { color grey3 } finish { silverish_finish } }
    global_settings {
     // max_trace_level 20
      #if (radio)
        //#debug "\nUsing 3.6 radiosity\n"
        radiosity {
          pretrace_start 0.08
          pretrace_end 0.01
          count 150
          nearest_count 7
          error_bound 0.1
          recursion_limit 1
          low_error_factor 0.7
          gray_threshold 0
          minimum_reuse 0.01
          brightness 1
          adc_bailout 0.01/2
        }
      #end
    }

  #break
#end
